# cython: language_level=3
# Cython file that provides an interface to C++ code

# Authors: Daniel Gribel, Lorenzo Saraiva and Thibaut Vidal
# Contact: dgribel@inf.puc-rio.br

import numpy as np
from libcpp.vector cimport vector

# distutils: language = c++
# distutils: sources = HGMeans.cpp

cdef extern from "HGMeans.h":
    cdef cppclass HGMeans:
        HGMeans()
        vector[vector[int]] Go(vector[vector[double]], vector[int], int, int, int, vector[int], int)
        void GoFile(char*, int, int, int, vector[int], int)

cdef class PyHGMeans:
    cdef HGMeans *thisptr      # hold a C++ instance which we are wrapping
    def __cinit__(self):
        self.thisptr = new HGMeans()
    def __dealloc__(self):
        del self.thisptr
    def Go(self, dataset, y, size_population, max_it, nb_it, list_clusters, w):
        return self.thisptr.Go(dataset, y, size_population, max_it, nb_it, list_clusters, w)
    def GoFile(self, filename, size_population, max_it, nb_it, list_clusters, w):
        self.thisptr.GoFile(filename, size_population, max_it, nb_it, list_clusters, w)
    def run(self, dataset, y, size_population=10, max_it=5000, nb_it=1, list_clusters=[2,3,5], w='0'):
        write = 0
        if w == 'w':
            write = 1
        return self.thisptr.Go(dataset, y, size_population, max_it, nb_it, list_clusters, write)
    def runfile(self, filename, size_population=10, max_it=5000, nb_it=1, list_clusters=[2,3,5], w='0'):
        py_filename = filename.encode('utf-8')
        write = 0
        if w == 'w':
            write = 1
        self.thisptr.GoFile(py_filename, size_population, max_it, nb_it, list_clusters, write)


from setuptools import setup, Extension

with open("README.md", "r") as fh:
    long_desc = fh.read()

setup(
    name = "hgmeans",
    version = "3.0",
    include_package_data = True,
    author = "Daniel Gribel",
    author_email = "dgribel@inf.puc-rio.br",
    url = "https://github.com/danielgribel/hg-means",
    description = "HG-means clustering for minimum sum-of-squares formulation",
    long_description = long_desc,
    # long_description_content_type = long_desc,
    keywords = "clustering optimization",
    classifiers = [
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python",
    ],
    ext_modules = [Extension(
            "hgmeans", [
                "hgmeans.pyx", 
                "hgmeans/HGMeans.cpp",
                "hgmeans/GeneticOperations.cpp",
                "hgmeans/MathUtils.cpp",
                "hgmeans/Solution.cpp",
                "hgmeans/hamerly/dataset.cpp",
                "hgmeans/hamerly/general_functions.cpp",
                "hgmeans/hamerly/hamerly_kmeans.cpp",
                "hgmeans/hamerly/kmeans.cpp",
                "hgmeans/hamerly/original_space_kmeans.cpp",
                "hgmeans/hamerly/triangle_inequality_base_kmeans.cpp"
            ],
            include_dirs=["hgmeans"],
            depends=["hgmeans/HGMeans.h"],
            language="c++",
            extra_compile_args=["-g", "-O3", "-std=c++11"]
    )]
)
